package cz.muni.fi.pv168.swingexample;

import com.sun.org.apache.bcel.internal.classfile.FieldOrMethod;
import java.awt.BorderLayout;
import java.awt.EventQueue;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.net.URL;
import java.util.List;
import java.util.concurrent.ExecutionException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.AbstractAction;
import javax.swing.Action;
import static javax.swing.Action.ACCELERATOR_KEY;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.SwingWorker;

public class SwingWorkerExample extends JFrame {

    private final JTextArea textArea;
    private final Action startMyOperationAction;
    private final Action cancelMyOperationAction;
    private final JProgressBar progressBar;
    private MyOperationSwingWorker curreMyOperationSwingWorker;
    
    public SwingWorkerExample() throws HeadlessException {
        super("SwingWorker Example");
        textArea = new JTextArea(25, 80);
        textArea.setEditable(false);
        
        Action exitAction = new AbstractAction("Exit", loadIcon("exit.png")) {
            
            {
                putValue(ACCELERATOR_KEY, 
                        KeyStroke.getKeyStroke(KeyEvent.VK_Q, ActionEvent.CTRL_MASK));
            }
            
            @Override
            public void actionPerformed(ActionEvent e) {
                System.exit(0);
            }
        };
        
        startMyOperationAction = new AbstractAction("MyOperation") {

            @Override
            public void actionPerformed(ActionEvent e) {
                curreMyOperationSwingWorker = new MyOperationSwingWorker();
                curreMyOperationSwingWorker.execute();
            }
        };
        
        cancelMyOperationAction = new AbstractAction("Cancel MyOperation") {
        
            {
                setEnabled(false);
            }
            
            @Override
            public void actionPerformed(ActionEvent e) {
                if (curreMyOperationSwingWorker != null) {
                    curreMyOperationSwingWorker.cancel(true);
                }
            }
        };

        JToolBar toolBar = new JToolBar();       
        add(toolBar, BorderLayout.NORTH);
        
        JMenuBar menuBar = new JMenuBar();
        setJMenuBar(menuBar);
        
        JMenu fileMenu = new JMenu("File");
        menuBar.add(fileMenu);

        fileMenu.add(startMyOperationAction);
        fileMenu.add(cancelMyOperationAction);
        fileMenu.addSeparator();
        fileMenu.add(exitAction);
        
        JButton button1 = new JButton(exitAction);
        add(button1, BorderLayout.SOUTH);
        
        toolBar.add(exitAction);
        toolBar.add(startMyOperationAction);
        toolBar.add(cancelMyOperationAction);
        
        add(new JScrollPane(textArea));
        
        progressBar = new JProgressBar();
        add(progressBar, BorderLayout.SOUTH);
        
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        pack();
    }

    
    private class MyOperationSwingWorker extends SwingWorker<Long, Integer> {

        public MyOperationSwingWorker() {
            startMyOperationAction.setEnabled(false);
            cancelMyOperationAction.setEnabled(true);
            progressBar.setValue(0);
            addPropertyChangeListener(e -> {
                if (e.getPropertyName().equals("progress")) {
                    progressBar.setValue((int) e.getNewValue());
                }
            });
        }
        
        @Override
        protected Long doInBackground() throws Exception {
            long sum = 0;
            for (int i = 0; i < 1_000; i++) {
                if (isCancelled()) {
                    return null;
                }
                setProgress(i / 10);
                publish(i);
                sum += i;
                Thread.sleep(10L);
            }
            return sum;
        }

        @Override
        protected void process(List<Integer> chunks) {
            for (Integer processedNumber : chunks) {
                textArea.append(String.format("Processing number: %d %n", processedNumber));
            }
        }

        @Override
        protected void done() {
            try {
                if (!isCancelled()) {
                    long result = get();
                    textArea.append(String.format("Sum is: %s %n", result));
                } else {
                    textArea.append(String.format("Operation cancelled"));
                }
            } catch (InterruptedException ex) {
                throw new AssertionError("Thread was interrupted unexpectedly", ex);
            } catch (ExecutionException ex) {
                Throwable originalException = ex.getCause();
                textArea.append(String.format("Error: %s %n",originalException));
            } finally {
                startMyOperationAction.setEnabled(true);
                progressBar.setValue(100);
                curreMyOperationSwingWorker = null;
                cancelMyOperationAction.setEnabled(false);                
            }
        }
        
    }
    
    private Icon loadIcon(String name) {
        URL iconUrl = SwingWorkerExample.class.getResource(name);
        if (iconUrl == null) {
            throw new IllegalArgumentException("Icon not found on classpath: " 
                    + SwingWorkerExample.class.getPackage().getName() + "." 
                    + name);
        }
        return new ImageIcon(iconUrl);
    }
    
    
    private static void showMainWindow() {
        new SwingWorkerExample().setVisible(true);
    }
    
    public static void main(String[] args) {
        EventQueue.invokeLater(SwingWorkerExample::showMainWindow);
    }
    
}
