package cz.muni.fi.pv168.swingexample;

import java.awt.BorderLayout;
import java.awt.EventQueue;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.net.URL;
import java.util.List;
import java.util.concurrent.ExecutionException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.AbstractAction;
import javax.swing.Action;
import static javax.swing.Action.ACCELERATOR_KEY;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.SwingWorker;

public class SwingWorkerExample2 extends JFrame {

    private final JTextArea textArea;
       
    private final Action startMyOperationAction;
    private final Action cancelMyOperationAction;
    
    private final JProgressBar progressBar;
    private MySwingWorker currentMySwingWorker;
    
    public SwingWorkerExample2() throws HeadlessException {
        super("SwingWorkerExample2");
        
        Action exitAction = new AbstractAction("Exit", loadIcon("exit.png")) {
            
            {
                putValue(ACCELERATOR_KEY, 
                        KeyStroke.getKeyStroke(KeyEvent.VK_Q, ActionEvent.CTRL_MASK));
                putValue(SHORT_DESCRIPTION, "Exit the application");

            }
            
            @Override
            public void actionPerformed(ActionEvent e) {
                System.exit(0);
            }
        };
        
        startMyOperationAction = new AbstractAction("Start MyOperation") {
            @Override
            public void actionPerformed(ActionEvent e) {
                new MySwingWorker().execute();
            }
        };
        
        cancelMyOperationAction = new AbstractAction("Cancel MyOperation") {
            
            {
                setEnabled(false);
            }
            
            @Override
            public void actionPerformed(ActionEvent e) {
                if (currentMySwingWorker == null) {
                    throw new IllegalStateException("No MySwingWorker is running");
                } else {
                    currentMySwingWorker.cancel(true);
                }
            }
        };
        
        JToolBar toolBar = new JToolBar();

        JMenuBar menuBar = new JMenuBar();
        JMenu fileMenu = new JMenu("File");
        menuBar.add(fileMenu);
                
        fileMenu.add(startMyOperationAction);
        fileMenu.add(cancelMyOperationAction);
        fileMenu.addSeparator();
        fileMenu.add(exitAction);
                
        toolBar.add(exitAction);
        toolBar.add(startMyOperationAction);
        toolBar.add(cancelMyOperationAction);
        
        add(toolBar,BorderLayout.NORTH);
        
        setJMenuBar(menuBar);
        
        JButton exitButton = new JButton(exitAction);
        
        add(exitButton,BorderLayout.SOUTH);
        
        textArea = new JTextArea(25, 80);
        textArea.setEditable(false);
        add(new JScrollPane(textArea));
        
        progressBar = new JProgressBar();
        add(progressBar, BorderLayout.SOUTH);
        
        pack();
        setDefaultCloseOperation(EXIT_ON_CLOSE);
    }

    
    private static Icon loadIcon(String name) {
        URL iconUrl = SwingWorkerExample2.class.getResource(name);
        if (iconUrl == null) {
            throw new IllegalArgumentException("Icon not found on "
                    + "classpath: " + SwingWorkerExample2.class.getPackage().getName() 
                    + "." + name);
        }
        return new ImageIcon(iconUrl);
    }
    
    private class MySwingWorker extends SwingWorker<Long, Integer> {

        @SuppressWarnings("LeakingThisInConstructor")
        private MySwingWorker() {
            assert EventQueue.isDispatchThread();
            if (currentMySwingWorker != null) {
                throw new IllegalStateException("Another worker is running");
            }
            currentMySwingWorker = this;
            startMyOperationAction.setEnabled(false);
            cancelMyOperationAction.setEnabled(true);
            progressBar.setValue(0);
            addPropertyChangeListener(e -> {
                if (e.getPropertyName().equals("progress")) {
                    progressBar.setValue((int) e.getNewValue());
                }
            });
        }
        
        @Override
        protected Long doInBackground() throws Exception {
            long sum = 0;
            for (int currentNumber = 0; currentNumber < 1_000; currentNumber++) {
                if (isCancelled()) {
                    return null;
                }
                publish(currentNumber);
                setProgress(currentNumber / 10);
                sum += currentNumber;
                Thread.sleep(2);
            }
            return sum;
        }

        @Override
        protected void process(List<Integer> processedNumbers) {
            for (int processedNumber : processedNumbers) {
                textArea.append(String.format("Processed: %d%n", processedNumber));
            }
        }

        @Override
        protected void done() {
            try {
                if (isCancelled()) {
                    textArea.append("Cancelled");
                } else {
                    long sum = get();
                    textArea.append(String.format("Result: %d%n", sum));
                    progressBar.setValue(100);
                }
            } catch (InterruptedException ex) {
                throw new AssertionError(ex);
            } catch (ExecutionException ex) {
                // Original exception thrown by doInBackground()
                Throwable originalException = ex.getCause();
            } finally {
                startMyOperationAction.setEnabled(true);  
                cancelMyOperationAction.setEnabled(false);
                currentMySwingWorker = null;
            }
        }        
        
    }
    
    
    
    private static void showMainWindow() {
        new SwingWorkerExample2().setVisible(true);
    }

    public static void main(String[] args) {
        EventQueue.invokeLater(SwingWorkerExample2::showMainWindow);
    }

}
